#!BPY

#"""
#Name: 'Cross Section'
#Blender: 243
#Group: 'Object'
#Tooltip: 'Creates a cross section of two meshes as a new edge'
#"""

####---Header------------------------------------------------------------------

__author__ = "Witold.Jaworski"
__url__ = ("http://www.blender.org","http://airplanes3d.net")
__email__ = ("me@wjaworski.pl")
__version__ = "1.00"
__bpydoc__ = """\
Finds the cross section between meshes of two selected objects. Creates an edge
from the cross section points.<br>
This edge is added to the active object mesh. It is not connected to other edges. 
  
Usage:<br>
1. Optional: in face select mode select a part of the mesh, that will be used in 
the intersection process. Otherwise whole mesh will be used, which may take 
more time.<br>
2. Select two objects;<br>
3. Select from menu Object-->Scripts-->Cross Section<br>
4. Fill fields in dialog box, that will appear, and press [OK];<br>
5. Result will be highlighted on the active object, in edit mode.

The found points will be placed in the mesh of the active object, connected
by edges, and finally selected (to let you find them easier).
You can do with the newly created edge what you want. It would be not too
difficult to remove some original mesh vertices and connect the cross section 
edges to other faces. This way you can quickly create holes, or sew two meshes.

Practical comment: script was tested also for special cases - when an edge from
one mesh has cross point exactly on an edge from the second mesh. In general,
such cases should be handled properly, but I cannot guarantee it in 100%.<br>
If you find, that script has not found some cross points - move one object a 
little bit, and try again. This would eliminate direct "edge to edge" hits. 
(When you are crossing two triangles, the typical edge has
 two points. Any other case is a special one. "Edge to edge" hits are the 
 most difficult to handle, because usually program does not know, where to 
 put, on the cross section edge, the third or fourth newly found point.)
"""
####Legal notoice-----------------------------------------------------------
# (c) 2007 Witold Jaworski (witold.jaworski@vaircraft.neostrada.pl)
# released under Blender Artistic License (www.blender.org)
####imports-----------------------------------------------------------------
from Blender import Mesh,Object
from Blender.Mathutils import Vector,Matrix,Intersect,CrossVecs,TriangleNormal
from Blender.Window import EditMode, WaitCursor
from Blender.Draw import Create,PupBlock,PupMenu
from time import time
####general comments-------------------------------------------------------
#The algorithm for the crossing calculation is following:
#1. Prepare the data - use meshes of selected objects to create two CMesh
#   instances. There no special assumptions about the topology of selected 
#   meshes - they can be a multi-piece mesh, closed or open.
#   Each CMesh stores a list of all faces of assigned object. They are converted
#   into world coordinates and eventually split from quads to triangles. They
#   are the CFace class instances. 
#	We will refer to the CMesh representing the first selected object as mesh A,
#   and the second as mesh B.  
#
#2. For each face from A: check intersection with every face of B faces. Store
#   the found cross points in A.cpoints list. The crossing points are searched
#   first crossing each A's edges (CEdge) with B face triangle. Then the B's 
#	  edges are crossed with A's face traingle. There should be always be a zero 
#   or two crossing points (CPoint) found. They are refered to each other, to 
#   allow arrange them into proper loop at the next stage. 
#   (They are found randomly, now, so it is not right place to discover the loop)
#
#3. Arrange the found cross points into properly ordered loops. To find a loop,
#   select the arbitrary (in implementation: first) point from list of found
#   points. Using the refernces created at 2. find all the ancestors of the 
#   selected point. Then find all precedensors, and connect this two lists
#   to get a complete loop.
#
#
#   In this implementation, to minimize the intersection calculations, first 
#   test is a "box" test: the extens of two faces are compared. Further 
#   calculations are performed only for the faces that have the "box" test 
#   passed.
#	  The face's edges (CEdge) are created "on demand", when there is a need to
#   cross them with a face. Many faces will never demand their edges, so they
#   will be never created. Edges, during creation, are referenced by both faces
#   that share it. 
#	  This will reduce greatly the number of edges, reducing also the storage.
#   Each edge "remembers" result of the crossing with each face, so also the
#   calculations with negative result will not be performed twice.
#   In general, this implementation uses heavily the memory, and minimizes
#   the CPU usage as much, as possible. 
#   
#   Below - definitions of four object classes, that take part in the interse-
#	  ction: 
#		CMesh, 
#		CFace, 
#		CEdge, 
#		CPoint (it is the result - cross point, not vertex!)
####constants---------------------------------------------------------------
EPSILON	= 0.000001       #the tolerance of crossing faces (in world units)
BOX_EPSILON = 0*EPSILON  #the tolerance of comparing 'boxes', containing
						 #faces. Actually it seems to have no influence
						 #on the final result.
DEBUG = 1 #Debug level. When nonzero, some diagnostics texts are placed at 
		  #output, during processing. Level = 1 generates some genral
		  #statements, Level = 5 - intermediate, while Level = 9 is the most 
		  #detailed
####globals-----------------------------------------------------------------

####classes-----------------------------------------------------------------

class CMesh:
	"""Represents a mesh, one of two used to find the intersection"""
	#public class fields 
	cpoints = [] 		#The result list of cross points (CPoint) found 
						#It is a reference to one global result list, so 
						#EVERY instance of this class will have access to
						#THE SAME result list (it resolved some algorithm
						#problems for printing the CPoint representation)	

	#private instance fields:
	#__src => Mesh:			the Blender's mesh object
	
	#	working dictionaries, used during intersection:
	#__verts => {Vector}: 	a dictionary of mesh vectors - in global space
	#						coordinates. The key is the Mesh vertex ordinal.
	#__edges => {Edge}:		dictionary of edges, that will be tested.
	#						The key is a tuple of begin and end vertices 
	#						ordinals. The smaller ordinal is always the first 
	#						element	of the tuple key.
	#						This is a working, partial list of Edge instances.
	#						Only edges that are "suspected" to have a crosss
	#						point with the second mesh are placed here.
	#						This "suspection" is based on spatial face's spatial
	#						position.
	#__neighbours=>{Face}	dictionary of faces, that forms the mesh. In this
	#						dictionary every face is registered 3 times. (for 
	#						every edge). The key is a tuple of two edge's 
	#						vertices ordinals. The sequence of these ordinals 
	#						matches the direction of the edge along assigned.
	#						Thus some of these keys will be a reversed version 
	#						of the keys in the __edgses dictionary, the others  
	#						will be identical.
	#						For all non-boundary edges there always are two keys,
	#						one for each face that shares this particular edge.
	#						The value tuples of such keys are a reversion of 
	#						each other.
	
	#public instance fields:
	#faces => {Face}		dictionary of faces, that forms the mesh. In this
	#						dictionary the key is a tuple containing 3 ordinals
	#						of face vertices. The sequence of this ordinals 
	#						describes the face direction.
	
	#instance methods:
	def __init__(self, object):
		"""Initializes a mesh instance
			Arguments:
				object: 	a Blender's object - a mesh object
			A ValueError will be thrown, when object has no mesh 
		"""
		if object.getType() != 'Mesh' : 
			raise ValueError("object '%s' has no mesh data" % object.name)
		self.__src = object.getData(mesh=True)
		self.__edges = {}
		self.__neighbours = {}
		self.faces = {}
		
		#fill the __verts dictionary (translate points to worldspace):
		self.__verts = {}
		m = object.getMatrix('worldspace')
		if DEBUG > 8: print "\nVertices %s:" % object.name
		for v in self.__src.verts:
			self.__verts[v.index] = v.co * m
			if DEBUG > 8:	print "%s: %s;" % \
										(repr(v.index), \
										repr(self.__verts[v.index]))				
		#register faces:
		selected = self.__src.faces.selected() #try to find the selected faces
		if not selected: selected = range(len(self.__src.faces)) #all
			
		for i in selected:
			f = self.__src.faces[i]
			if f.area > EPSILON:
				keys = [v.index for v in f.v] #3 or 4 vertices
				if len(keys) == 3: #a triangle - no cut needed
					self.AddFace(f, (keys[0],keys[1],keys[2]))
				elif len(keys) == 4: #a quad - cut it into two triangles
					#l0, l2: the lengths fo the dialgonals
					l0 = self.vert(keys[2]) - self.vert(keys[0])
					l2 = self.vert(keys[3]) - self.vert(keys[1])
					if l2.length > l0.length: #cut along (1, 3) edge
						self.AddFace(f, (keys[0],keys[1],keys[3]))
						self.AddFace(f, (keys[1],keys[2],keys[3]))
					else: #cut along (0, 2) edge
						self.AddFace(f, (keys[0],keys[1],keys[2]))
						self.AddFace(f, (keys[0],keys[2],keys[3]))
						
	def name(self):
		"""Returns the name of wrapped Blender Mesh object
			(useful for diagnostic messages)
		"""		
		return self.__src.name
	
	def vert(self, index):
		"""Returns one of the mesh vertices (a Vector, in world coordinates)
			Arguments:
				index:	the vertex index in the mesh 
		"""
		return self.__verts[index]

	def neighbour(self, edgekey):
		"""Returns the face that uses the same edge, but in opposite direction 
			Arguments:
				edgekey:	the key of the edge, along face direction (i.e.
							NOT normalized: as it is in CFace.__edges dictionary
		"""
		#the neighbour HAS to be registered in __neighbours under the 
		#reversed key:
		key = (edgekey[1],edgekey[0])
		if self.__neighbours.has_key(key): return self.__neighbours[key]
		else: return None
		
	def cpindex(self,cpoint):
		"""Returns index of given cross point (CPoint), or None if not registered
			Arguments:
				cpoint:	a cross point (a CPoint instance) that is chcecked
			
			Used for diagnostics and debbuging, not especially fast.
			For some cross points can return None, because they have been
			discarded during calculations.
		"""
		if cpoint in self.cpoints : return self.cpoints.index(cpoint)
		else: return None
		
	def GetEdge(self,key):
		"""Returns an edge (CEdge instance) that matches the given key
			Arguments:
				key:	a tuple of 2 integers - vertex ordinals
		"""
		key = CEdge.Normalize(key) #the key may come in non-canonical order
								   #(smaller ordinal as the second element)
								   #so we have to get rid of it.
		if self.__edges.has_key(key): 
			return self.__edges[key]
		else:
			index = self.__src.findEdges(key[0], key[1])
			if index: #this edge exists in source mesh:
				edge = CEdge(self, self.__src.edges[index], key, \
										self.vert(key[0]), self.vert(key[1]))
			else: #no edge found - this is a diagonal temporary edge:
				edge = CEdge(self, None, key, \
										self.vert(key[0]), self.vert(key[1]))

			#add the edge to the edge dictionary:
			self.__edges[key] = edge

			#assign it to two neighbour faces:
			fcount = 0 #count of the faces that share this edge
			if self.__neighbours.has_key(edge.key):
				self.__neighbours[edge.key].Assign(edge)
				fcount += 1

			 #The other face is registered at inverted the key tuple:
			if self.__neighbours.has_key(edge.altkey()):
				self.__neighbours[edge.altkey()].Assign(edge)
				fcount += 1
				
			#edge is a boundary edge, when it is not shared by two faces:
			edge.boundary = (fcount < 2)

			return edge
	
	def AddFace(self, mface, key):
		"""	Appends to faces, __neighbours dictionary  (of CFace instances) 
			the newly created face, based on source mesh
			Arguments:
				mface:	a source data - the MFace (may be a quad)
				key:	a tuple of 3 integers - vertex ordinals
		"""
		face = CFace(self, mface, key)
		self.faces[key] = face
		
		#in the __neighbour dictionary every face is represented 3 times-
		#by its edges oridnals:
		for k in [(key[0],key[1]),(key[1],key[2]),(key[2],key[0])]:
			self.__neighbours[k] = face
			
	def Intersect(self, key, face):
		"""Intersects the face from this mesh, identified by a key, with another
			Arguments:
				key:	tuple (3 integers - vertex ordinals) that identifies 
						a face in this mesh (in faces dictionary)
				face:	a face from another mesh (CFace)
			
			Returns a list of 2 cross points, or empty list
		"""
		a,b = self.faces[key],face #a, b are just a shortcuts for the full names
		result = a.Intersect(b)	#store in the list the cross points from a...
		result.extend(b.Intersect(a)) #... together with b.
		if result:
			if DEBUG > 8:	print "\n%s %s x %s %s (%d points):" % \
										(a.mesh.name(), repr(a.key), \
										 b.mesh.name(),repr(b.key),len(result))
										
			#Normally there should be always 2 cross points in the result,
			#but sometimes, when edge from one mesh crosses exactly the edge 
			#from the other mesh, there may appear 1 or 3 points. There may
			#be also, theoretically, 4 (two triangles crossing each other edges)
			#but I have never seen such case. (I have try to obtain it, but
			#there always were 3, 2 or 1 points). 
										
			if len(result) == 4: #I have never seen such case 
				if DEBUG:
					print "4 cross points at crossing %s (%s) with %s (%s)" % \
						  (self.name(), repr(key), face.mesh.name(), repr(face.key))											
			elif len(result) == 3:	#3 cross points
				#points 0 and 1 or 1 and 2 are from the same face:
				#(because we first cross edges A with face B, and 
				#then face A with edges B).
				#
				#Heuristics: put the single point from the other face
				#between the two that belong to the same face.
				#(If we will not give this way a chance to this single
				#additional edge to be assigned, they will often break
				#the prev:next sequence, used to find topological sequence
				#of the found cross points):
				if result[0].edge.mesh is result[1].edge.mesh:
					#points 0 and 1 are from the same face:
					r = result[0].vert() - result[1].vert()
					if r.length > EPSILON: #put this single in the middle:
						result = [result[0],result[2],result[1]]
				else:
					#points 1 and 2 are from the same face:
					r = result[1].vert() - result[2].vert()
					if r.length > EPSILON: #put this single in the middle:
						result = [result[1],result[0],result[2]]
			else: pass #len in 1,2: nothing special

			if len(result) > 1: #we will not append the single point to results
			
				#assign the found points into a topological chain:
				prev = result[0]
				for cp in result[1:]:
					prev.Assign(cp)
					cp.Assign(prev)
					prev = cp
					
				#Special case, when we have 3 cross points:
				#we will try to start collecting data from it.
				#So - let's put it into the unusual place: as the first
				#because the getLoop() function always start with the first
				#all the other points are placed at the end of cpoints[]
				if len(result) == 3: 
					cp = result[1]
					if cp not in self.cpoints: self.cpoints.insert(0,cp)
					#because result[1] is already on the list, it will be 
					#discarded in the loop below, and only the items from
					#the same face will be added at the end of cpoints...

				#store the normal pair in the cpoints list:p
				for cp in result:
					if cp not in self.cpoints: self.cpoints.append(cp)
							
			if DEBUG > 8: #beware tab position: this will print also the discarded ones 
				for cp in result: print cp

		return result
	
class CFace:
	"""Represents a triangle face, element of the mesh"""
	#instance fields:
	#src => MFace:		the Blender's face object. Because it may be a 
	#					quad - two CFaces may refer to the same MFace.
	#					They will differ by the key value
	#mesh =>CMesh		the parent object, that:
	#								 	(mesh.faces[self.key] is self) == True
	#key => (3 int):	the face id (matches the key in CMesh.faces dictionary).
	#					It is a tuple of face vertices ordinals. It determines
	#					direction of the face.
	#min =>[3 floats]:	minimum x,y,z face coordinates
	#max =>[3 floats]:	maximum x,y,z face coordinates
	#
	#	private fields
	#__edges=>{3 Edges}:a dictionary of 3 Edge instances. The face edges.
	#					The key of this dictionary is a tuple of 2 vertex
	#					ordinals. It should match a key in the parent 
	#					CMesh.__neigbours dictionary key.
	#					At the begining all elements are set to None. It is 
	#					filled with objects when face is suspected to have 
	#					a cross section with a face from the other mesh.
	#__boundary=>Bool:	True, when face contains a boundary edge. Used by
	#					HasBoundaryEdge() function, calculated on first demand
	#__normal=>Vector:	normal to this face. Used by normal() function, calcu-
	#					lated on first demand
	def __init__(self, cmesh, mface, key):
		"""Creates a new traingular face, representing a part of a MFace
			Arguments:
				cmesh:	parent object (CMesh)
				mface:	a source face, from the original mesh (MFace)
				key:	a tuple of 3 vertex ordinals. They should be
						contained in mface.ed
		"""
		self.src = mface
		self.mesh = cmesh
		self.key = key
		v0,v1,v2 = self.vert(0),self.vert(1),self.vert(2)
		self.min = [min(v0.x,v1.x,v2.x)-BOX_EPSILON, \
					min(v0.y,v1.y,v2.y)-BOX_EPSILON, \
					min(v0.z,v1.z,v2.z)-BOX_EPSILON]
		self.max = [max(v0.x,v1.x,v2.x)+BOX_EPSILON, \
					max(v0.y,v1.y,v2.y)+BOX_EPSILON, \
					max(v0.z,v1.z,v2.z)+BOX_EPSILON]
		#__edges - empty places, for begining. Just to keep their keys
		self.__edges = {}
		for k in [(key[0],key[1]),(key[1],key[2]),(key[2],key[0])]:
			self.__edges[k] = None
			
		#currently unused:
		self.__boundary = None #will be calculated on first demand
		self.__normal = None #will be calculated on first demand
		
	def vert(self, index):
		"""Returns one of the face's vertices (a Vector, in world coordinates)
			Arguments:
				index:	the vertex index on the face (0, 1, or 2)
			
			The index==i selects the mesh vertex having ordinal==face.key[i] 
		"""
		return self.mesh.vert(self.key[index])

	def Assign(self, edge):
		"""Assigns an edge to the face
			Arguments:
				edge:	an edge (CEdge), that has be placed in previously 
						empty place in face's edge dictionary
		"""
		key = edge.key
		#one of two faces that uses this edge has it under reversed tuple key:
		if not self.__edges.has_key(key): key = edge.altkey()
		
		if self.__edges[key] is None:
			self.__edges[key] = edge
		elif self.__edges[key] is not edge: 
			if DEBUG > 5:
				print "Attempt to reassign edge %s, in face %s" % \
													(repr(key), repr(self.key))

	def Intersect(self, face):
		"""Intersects a face with another face, from different mesh
			Arguments:
				face:	a face (CFace) from different mesh, to be tested
			
			Returns a list of found crossed points (CPoints). The list may be 
			empty, or contain 1 or 2 points. Points found are the intersections
			of this face EDGES with the surface of the other face, ONLY!
		"""
		#first, discard operation when the spatial limits are separate:
		for i in range(0,2):
			if (self.min[i] > face.max[i] or self.max[i] < face.min[i]): 
				return [] #no cross points
		#if the faces are suspected to have a cross point - check every edge:
		cpoints = []
		for k in self.__edges.keys():
			if not self.__edges[k]: self.__edges[k] = self.mesh.GetEdge(k)
			result = self.__edges[k].Intersect(face)
			if result: cpoints.append(result)
			
		return cpoints

	
class CEdge:
	"""Represents an edge - element of the mesh"""
	#instance fields:
	#src => MEdge:		the Blender's edge object. This field may be == None
	#					for the diagonal edges, created solely for this 
	#					calculation for quad MFaces. Such temporary diagonal
	#					join two traingle Faces. The src attribute of these
	#					faces refers to the same quad MFace.
	#mesh =>CMesh		the parent object, that:
	#								 (mesh.__edges[self.key] is self) == True
	#key => (2 int):	the edge id (matches the key in CMesh.__edges dictionary).
	#					It is a tuple of edge vertices ordinals. It determines
	#					direction of the edge.
	#org => Vector:		begining of the edge (world coordinates)
	#ray =>	Vector:		a vector from begining to the end of the edge
	#boundary => Bool:	True, when it is a boundary edge.
	#
	#private instance fields:
	#__results=>{CPoint}: a dictionary of calculation results, that were already
	#					performed. Its keys are the keys of the tested faces.
	#					Preasumably, most of the values will be None.
	def __init__(self, mesh, medge, key, v1, v2):
		"""Creates a new edge
			Arguments:
			mesh:	the mesh (CMesh) that contains this edge
			medge:	the source edge (a MEdge) - may be None for diagonals
			key:	the tuple of two mesh vertex ordinals (integers)
					that is used as a key in parent's CMesh.__edges dictionary
			v1:		the begining of the edge (a Vector, in worldspace)
			v2:		the end of the edge (a Vector, in worldspace)
			Be sure, that v1 is really the vertex(key[0]), and v2 - vertex(key[1])
		"""
		self.mesh = mesh
		self.src = medge
		self.key = key
		self.org = v1
		self.ray = v2 - v1
		self.__results = {}
		self.boundary = False
		
	@staticmethod
	def Normalize(key):
		"""Returns a canonical form of a edge key - smaller ordinal first
			Arguments:
				key:	a tuple of two vertex ordinals. They may be in any
						order
			
			This function is used by CMesh to unify the edge keys, requested
			in a reversed form by faces that share the edge.
		"""
		if key[0] < key[1]:	return key
		else: return (key[1],key[0])
	
	def altkey(self):
		"""Returns an alternated (reversed) form of the edge key.
		Required by CFace, because it stores their edges in a non-canonical
		form. The order of vertex ordinals in keys of their local edge dictionary
		reflects the local face direction, not the normalized edge key. 
		"""
		return (self.key[1], self.key[0])

	def Intersect(self, face):
		"""	Calculates the intersection point of this edge and a face.
			Arguments:
				face:	a triangular face (CFace), that belongs to another mesh
				
			Returns a CPoint, if suceeds, or None, when there is no cross point.
			Stores the result in the internal cpoints dictionary.
		"""
		if self.ray.length < EPSILON : return None #this edge hardly exists!
		#if we have already meet this face: return the result
		if self.__results.has_key(face.key): 
			return self.__results[face.key]
			#if p is None or p.disconnected(): return None #skip over discarded points
			#else: return p
		else: #it has been not tested, yet
			result = None 
			p = Intersect(face.vert(0),face.vert(1),face.vert(2), \
														self.ray, self.org, 1)
			if p:
				v = (p - self.org) #result vector
				if v*self.ray > 0: #the ray and the result in the same direction
					#is the point inside the segment?
					if v.length <= (self.ray.length + EPSILON) : 
						result = CPoint(self,face, v.length/self.ray.length)
			#we will also remember the cases which have failed:
			#(just to not repeat them:)
			self.__results[face.key] = result
			return result
		
class CPoint:
	"""Represents a cross point of an edge of mesh A and a face of mesh B"""
	#instance fields:
	#edge => Edge:		the edge that contains this cross point
	#face => Face:		the face that contains this cross point
	#					Beware: the edge the face always belong to different
	#					CMeshes!
	#t => float:		the location on the edge (0.0-at the begining, 1.0-at
	#					the end)
	#prev: CPoint		previous point on the crossing line
	#next: CPoint		next point on the crossing line
	#
	#Beware: during first phase of the calculations - finding the crossing 
	#points, some points may have the next or prev field set to None.
	#The general arrangement of prev->prev->prev or next->next->next between
	#the cross points may be reversed in some places. This is checked and fixed 
	#during second phase of the calculations - the cross point arrangement -
	#by the getTail() global function.
	#In fact, the names "prev" and "next" are somewhat improper, They are,
	#in fact, the references to the neighbours, but the direction is uknown for
	#most of the CPoint instance's "life".
	def __init__(self, edge, face, t):
		"""Creates a new instance of cross point
			Arguments:
			edge:	the edge that has this cross point (a CEdge)
			face:	the face that has this cross point (a CFace)
			t:		placement of the cross point (float: 0.0 means
					'at the begining of the edge', 1.0 means 'at the end')
		"""
		self.edge = edge
		self.face = face
		self.t = t
		self.next = self.prev = None #it will be set later
	
	def index(self):
		"""Returns an ordinal of this cross point, or None for not registered
			Use for diagnostics or debugging. Some points can be not registered
			in the prent CMesh.cpoints list, because they duplicated the ones
			that are already registered.
		"""
		return self.edge.mesh.cpindex(self)
		
	def __repr__(self):
		"""Represents content of the instance in a human-readable form
			For the diagnostics and debugging purposes
		"""
		#some shortcuts, first
		v = self.vert()
		if self.prev : p = self.prev.index()
		else: p = None
		if self.next : n = self.next.index()
		else: n = None
	
		#and finally - the result
		return 	"%s: (%1.2f, %1.2f, %1.2f) at %1.2f of '%s' %s x '%s' %s, btw:(%s, %s)" % \
					(repr(self.index()), v.x, v.y, v.z, self.t, \
					 self.edge.mesh.name(), repr(self.edge.key), \
					 self.face.mesh.name(), repr(self.face.key), \
					 repr(min(p,n)),repr(max(p,n)))
	
	def Assign(self, cpoint):
		"""Assigns a cpoint at free connection point (prev or next)
			Arguments:
				cpoint:	another cross point (CPoint) that has to be connected
		"""
		if self.next is cpoint or self.prev is cpoint: return #already done
		if not self.next: self.next = cpoint
		elif not self.prev: self.prev = cpoint
		elif DEBUG > 5:
			#sometimes it may happen, when edge from one mesh crosses exactly
			#edge from the other mesh 
			print "Attempt to reassign %s, already assigned to (%s, %s), to %s" % \
									(repr(self.index()), repr(self.prev.index()), \
									 repr(self.next.index()), repr(cpoint.index()))
		else: pass
	
	def GetOpposite(self, cpoint):
		"""Returns an opposite cross point to the given one
			Arguments:
				cpoint:	one of two cross points: next or prev
		"""
		if cpoint is self.next : return self.prev
		elif cpoint is self.prev: return self.next
		else:	
			if DEBUG > 5: #can happen for points that cross an edge from second mesh
				print "Cross point %d is not referenced by %d" % \
												(self.index(), cpoint.index())

	def vert(self):
		"""Returns the location of this point (a Vector, in worldspace)"""
		return self.edge.org + (self.t* self.edge.ray)
	
	def IsTemporary(self):
		"""	Returns True, when this point lies on a temporary edge
		
			Some edges of the CMesh do not exists in the source Mesh - they
			are face diagonals, artifically created on quad faces. Such edges
			are temporary - created only for this crossing calculation. 
			Such cross points are usually not wanted in the final result.
		"""
		return (self.edge.src is None) #such edges do not have the source 
									   #MEdge assigned.
	
	def BelongsTo(self, cmesh):
		"""	Returns True, when this point belongs to this mesh
			Arguments:
				cmesh:	mesh (CMesh) to be tested
			
			Cross point belongs to the mesh, that is the owner of the edge that
			has been crossed, not the face.
		"""
		return (self.edge.mesh is cmesh)

####module functions--------------------------------------------------------
def show(msg,title = "Mesh Intersection"):
	"""Shows a popup message
		Arguments:
			msg:	message text
			title:	optional. Non-standard title
	"""
	PupMenu(title + "%t|" + msg)
	
def getTail(cpoints, cpoint):
	"""Returns a sequence of cross points, that begins with the given point
		Arguments:
			cpioints:	the list of the crosspoints that will be searched;
			cpoint:		the the begin of the sequence - MUST NOT be in cpoints,
						but cpoint.next or cpoint.prev - should be
		
		This function returns a list containg all successors of the cpoint,
		that are - topologically - on the same side.
		The side does not need to be specified in parameters, because this
		function is	removing from cpoints every element, that is placed in 
		the result list.
		getTail() always tries to find the sequence that begins with the 
		cpoint.next. When you will call it second time - it has no choice, but
		return the sequence that begins with cpoint.prev. 
	"""
	result = []
	next = None #if cpoint has no next nor prev in cpoints, this will ensure,
				#that an empty list will be the result
	
	#check out, which direction we will take:
	if cpoint.next in cpoints: 		next = cpoint.next
	elif cpoint.prev in cpoints: 	next = cpoint.prev
	
	#the cross points reference each other via next and prev without any
	#special order (because they were found randomly). It may happen that
	#the cpoint.prev is the cpoint.prev.prev - when two neighbour cross points
	#have been created at reverse. One thing is sure: the opposite cross point
	#is one of the prev,next pair, so we have utilize the GetOpposite() function
	#to obtain from the next a point that is not the cpoint!
	while next in cpoints:
		cpoints.remove(next) #ensure, that this point will not be used again.
		
		#discard multipled points (they may happen in regular shapes, where 
		#cross points are exactly on face's boundary. For example: crossing of  
		#two identical cubes, where one was moved away by half of their size) 
		#if (next.vert()-cpoint.vert()).length > EPSILON: 
		result.append(next)		
		#proceed to the next one: 	
		#give me the next that is on the opposite side than the one (cpoint)
		#that we just have came, and mark actual next as the cpoint:
		cpoint,next = next,next.GetOpposite(cpoint)
	return result

def getLoop(cpoints):
	"""Returns from the cpoints points, that forms a continous sequence
		Arguments:
			cpoints: the sequence (a list of CPoints) to be sought
			
		This function removes every matching point from the CPoints seqence, and
		places it in the result list. When it has finished its run, the cpoints
		sequence is empty or contains a bunch of points that belong to another 
		loop. You should repeat call to this function, until you will collect
		all the loops.
	"""
	result = []
	if cpoints : #if there is anything to do:
		point = cpoints[0] #we will take an arbitrary point to start with:
		cpoints.remove(point) 
		result.append(point)
		left = (getTail(cpoints,point)) # the "next" part of the loop
		right = getTail(cpoints,point) # the "prev" part of the loop
		
		if left : result.extend(left)
		if right: 
			right.reverse()
			right.extend(result)
			result = right

	return result

def intersect(A,B):
	"""Calculates intersection of two mesh objects
		Arguments:
			a:	the first mesh (CMesh)
			b:	the second mesh (CMesh)
		
		Returns a list of the cross points loops found. Every element is a list
		of cross points (CPoint) objects, that represent a single topological 
		loop. These loops may be opened or closed.
	"""
	for i in A.faces.keys():
		for j in B.faces.keys():
			A.Intersect(i, B.faces[j])
	result = []		
	cpoints = A.cpoints[:]
	while cpoints:
		result.append(getLoop(cpoints))
	return result

def create(loop, object, allEdges=False, omit=None):
	"""creates a new edge loop in the mesh of given object
		Arguments:
			loop:		list of cross points (CPoint)
			object:		object, which mesh will be used to add the cross edge
			allEdges:	optional. Use all cross points, also the temporary 
						points that	belong to diagonals of quad faces (Boolean)
			omit:		optional. Cross points that lie on edges of this CMesh
						should be omitted during drawing
	"""
	m = object.getMatrix('worldspace').copy()
	m.invert() #transformation to object's space
	mesh = object.getData(mesh=1) #mesh, that will be extended by the new loop
	verts = [] #temporary list of vertices (Vector, object's local space)
	edges = [] #temporary list of edges = two element lists of vertex ordinals
	prev = None #helper: a previously processed CPoint.
	i = 0	    #actual vertex number
	base = len(mesh.verts) 	#ordinal of the first vertex, that will be added
							#to the mesh. (We have use the absolute ordinals)
	for cp in loop:
		#if it is not a point on a non-existant edge, or from mesh to be omitted:
		if 	(not cp.IsTemporary() or allEdges) and not cp.BelongsTo(omit):
			if prev is None: #Init: we are looking for the first valid point
				r = loop[-1].vert() - cp.vert() #distance from loop ends
												#it is = 0 for one element list
				if r.length > EPSILON: #if end and start point do not overlap:
					verts.append(cp.vert()*m)
					prev = cp
					if DEBUG > 5: print cp 
				else: pass #loop ends overlap, proceed to the next cp
			else: #Normal: prev exists!
				r = prev.vert() - cp.vert() #distance from the previous point
				if r.length > EPSILON:
					verts.append(cp.vert()*m)
					i += 1
					edges.append([base+i-1, base+i])
					prev = cp
					if DEBUG > 5: print cp
	
	if len(verts) > 1: #if there is sany segment to create:
		#if the loop is a closed loop: add closing edge
		if not (loop[0].edge.boundary or loop[-1].edge.boundary):
			edges.append([base+i, base]) #add the closing segment
		#and, finally, create
		mesh.verts.extend(verts)
		mesh.edges.extend(edges)

def selectOptions(A, B, useBFaces, useDiagonals):
	"""Displays the options modal dialog for this program and return result
		Arguments:
			A:				first face (a CMesh)
			B:				second face (a CMesh)
			useBFaces:		an Button object (created with Create() function)
			useDiagonals:	an Button object (created with Create() function)
		Returns True, when the user has accepted this operation
		User can change the values of useBFaces, useDiagonals objects.
		They should be verified after calling this function, if it returns True.
	"""
	content=[("%d triangle faces from A:" % len(A.faces)), \
			 (A.name()), \
			 ("%d triangle faces from B:" % len(B.faces)), \
			 (B.name()), \
			 ("Use B edges", useBFaces, \
			  "Points from both meshes will create the result edge (if not - A mesh only)"), \
			 ("Use diagonals", useDiagonals, \
			  "Use also points from temporary diagonals that divide the quad faces")]
	return PupBlock("Mesh Intersection",content)
	

def main():
	"""main function"""
	#prepare data selected by the user:
	sel = Object.GetSelected()
	if len(sel) != 2:
		show("Select two objects - actually selected: %d" % len(sel))
		return

	if DEBUG : 
		print "\nObjects that will be crossed: \n%s \nand \n%s." % \
												(sel[0].name,sel[1].name)
		
	#create data structures:
	try:
		A = CMesh(sel[0])
		B = CMesh(sel[1])
	except ValueError: #signalled, when one of the objects is not a mesh
		show(inst.args)
		return
	#the other exceptions - signal as usual...
	
	#show user options dialog:
	useBfaces = Create(1)
	useDiagonals = Create(0)
	if not selectOptions(A,B,useBfaces,useDiagonals): 
		pass #return #cancelled by the user

	#otherwise: analyze the user response....
	if useDiagonals.val: allEdges = True
	else: allEdges = False
	if useBfaces.val: omittedMesh = None
	else: omittedMesh = B
	
	#....and proceed:
	start = time()
	WaitCursor(1)
	result = intersect(A,B)
	if DEBUG: 
		seconds = time() - start
		print "\nIn %1.5f seconds created: %d edges, found: %d raw cross points\n" % \
									(seconds, len(A._CMesh__edges), len(A.cpoints))
	if DEBUG > 8: #let's look at the results:
		print "Raw result list:"
		for p in A.cpoints:
			print p
		
		print "\nResult:"

	mesh = sel[0].getData(mesh=1)
	i = 0
	base = len(mesh.verts)
	EditMode(0) #enter normal mode...
	for l in result:
		i += 1
		if DEBUG > 5: 
			print "Loop %d" % i
			print "  all points ...."
			for p in l : print p
			print "  ... and points selected to create the edge:"
		create(l, sel[0], allEdges, omittedMesh)
		
	
	mesh.sel = 0 	#deselect all previously selected vertices...
	#and select the ones that belong to the newly created loops:
	for v in mesh.verts:
		if v.index >= base: v.sel = 1

	WaitCursor(0)
			
	EditMode(1) 

main()
